<?php
namespace Estoque\Repositories;

class Itlanctoest extends \Doctrine\ORM\EntityRepository
{

    const STATUS_N = 'N';

    /**
     * Verifica se há alguma retirada sem nota fiscal.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtprocesso
     *
     * @throws \Exception
     * @return mixed
     */
    public function verificaRetiradaSemNotas($nrorg, $filial, $dtprocesso){
        $sql = "SELECT C.*
                  FROM RETIRADA R, ITEMRETI A, ITLANCTOEST C
                 WHERE R.NRORG        = :P_NRORG
                   AND R.CDFILIAL     = :P_CDFILIAL
                   AND R.NRORG        = A.NRORG
                   AND R.CDFILIAL     = A.CDFILIAL
                   AND R.NRRETIRADA   = A.NRRETIRADA
                   AND R.NRCARDAPIO   = A.NRCARDAPIO
                   AND A.NRORG        = C.NRORG
                   AND A.CDFIITLANEST = C.CDFILIAL
                   AND A.NRLANCESTQ   = C.NRLANCESTQ
                   AND A.NRSEQUITEM   = C.NRSEQUITEM
                   AND C.NRORG        = A.NRORG
                   AND C.CDFILIMOVI   = A.CDFIITLANEST
                   AND C.DTLANCMOVI   = :P_DATAPROC
                   AND C.IDEVOLESTOQ  = :P_STATUS_N";

        $params = array(
            "P_NRORG"    => $nrorg,
            "P_CDFILIAL"   => $filial,
            "P_DATAPROC" => $dtprocesso,
            "P_STATUS_N" => self::STATUS_N,
        );
        try{
            return self::getEntityManager()->getConnection()->fetchColumn($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
        /*$entity_manager = $this->getEntityManager();
        $result_set_mapping = new \Doctrine\ORM\Query\ResultSetMappingBuilder($entity_manager);
        $result_set_mapping->addRootEntityFromClassMetadata('\Estoque\Entities\Itlanctoest', "C");

        $result = $entity_manager->createNativeQuery($sql, $result_set_mapping)->execute($params);
        return count($result) > 0 ? $result : false;*/
    }

    /**
     * Verifica retirada de estoque
     *
     * @param unknown $filial
     * @param unknown $dtprocesso
     * @throws \Exception
     * @return mixed
     */
    public function verificaRetirada($filial, $dtprocesso){
    	$sql = "SELECT *
				FROM RETIRADA
				WHERE DTRETIRADA = :P_DATAPROC
					  AND CDFILIAL = :P_CDFILIAL";

    	$params = array(
            "P_DATAPROC" => $dtprocesso,
            "P_CDFILIAL"   => $filial
    	);
        try{
            return self::getEntityManager()->getConnection()->fetchColumn($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }


    public function consultaAjustesEstoque($nrorg, $filial, $dtInicio, $dtFinal){
        $sql = "SELECT MIN(DTLANCMOVI) AS DTLANCMOVI,
                       MAX(DTLANCMOVI) AS DTLANCMOVIF,
                       MAX(IDORDEMCLC) AS IDORDEMCLC,
                       MAX(CDARVPROD) AS CDPRODFIN,
                       MIN(CDARVPROD) AS CDPRODINI
                 FROM ITLANCTOEST I, PRODUTO P
                WHERE NRORG = :P_NRORG
                  AND CDFILIMOVI  =  :P_CDFILIAL
                  AND DTLANCMOVI BETWEEN :P_DTINICIAL AND :P_DTFINAL
                  AND I.CDPRODUTO = P.CDPRODUTO
                  AND IDTIPOMOVI IN ('0','1')";

        $params = array(
            "P_NRORG"     => $nrorg,
            "P_CDFILIAL"  => $filial,
            "P_DTINICIAL" => $dtInicio,
            "P_DTFINAL"   => $dtFinal,
        );

        $entity_manager = $this->getEntityManager();
        $result = $entity_manager->getConnection()->fetchAll($sql, $params);

        return count($result) > 0 ? $result : false;
    }

    /**
     * Altera o id de evolucao de estoque para uma lista de itens do lancamento de estoque, a partir do lancamento de
     * estoque.
     *
     * @param $cdfilial
     * @param $nrlancestq
     * @param $nrorg
     * @param $idevolestoq
     * @throws \Exception
     */
    public function evoluiEstoquePorLancamento($cdfilial, $nrlancestq, $nrorg, $idevolestoq)
    {
        $sql = "UPDATE ITLANCTOEST
                  SET IDEVOLESTOQ  = :IDEVOLESTOQ
                  WHERE CDFILIAL   = :CDFILIAL
                    AND NRLANCESTQ = :NRLANCEST
                    AND NRORG      = :NRORG ";

        $params = array(
            "IDEVOLESTOQ" => $idevolestoq,
            "CDFILIAL"    => $cdfilial,
            "NRLANCEST"   => $nrlancestq,
            "NRORG"       => $nrorg,
        );

        try {
            $this->getEntityManager()->getConnection()->executeUpdate($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificaMovimentacaoEstoqueParaProduto($filial, $produto)
    {
        $sql = <<<SQL
          SELECT COUNT(*) NRMOVIMENTA
          FROM ITLANCTOEST
          WHERE CDFILIMOVI = :P_CDFILIAL
          AND CDPRODMOVI = :P_CDPRODUTO
          AND NVL(IDEVOLESTOQ, 'S') = 'S'
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_CDPRODUTO" => $produto,
        );

        try{
            return self::getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function verificaMovimentacaoParaFilial($filial, $organizacao)
    {
        $sql = <<<SQL
        SELECT MIN(DTLANCMOVI) DTMENOR, MAX(DTLANCMOVI) DTMAIOR
           FROM ITLANCTOEST
        WHERE CDFILIMOVI = :P_CDFILIAL
          AND IDEVOLESTOQ <> 'N'
          AND NRORG = :P_NRORG
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRORG" => $organizacao,
        );

        try{
            return self::getEntityManager()->getConnection()->fetchArray($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function consultaProximoCodigoItem($filial, $lancamento)
    {
        $sql = <<<SQL
            SELECT NVL(LPAD(TO_CHAR(MAX(NRSEQUITEM) + 1),4,'0'), '0001')
            FROM ITLANCTOEST
            WHERE CDFILIAL = :P_CDFILIAL
            AND NRLANCESTQ = :P_NRLANCESTQ
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRLANCESTQ" => $lancamento,
        );

        try{
            return self::getEntityManager()->getConnection()->fetchArray($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
}